from pyjamas.ui.ScrollPanel import ScrollPanel
from pyjamas.ui.SimplePanel import SimplePanel
from pyjamas.ui.Tree import Tree
from pyjamas.ui.TreeItem import TreeItem
from pyjamas import DOM
from pyjamas import Window
from pyjamas.ui.RootPanel import RootPanel

"""This is a tree building example.  It has several objectives.
   run it to see what they are."""


MAXLEVELS = 10
USER = 'User'
class TTS(object):
    def __init__(self, name, child=[]):
        self.name = name
        self.child = child
    def isLeaf(self):
        return not self.child

class Error(Exception):
    pass

class BeyondDesignLimitError(Error):
    pass  ## The reason for this is good programming practice make py errors
          ## useful

class TreeDemo(ScrollPanel):

    def __init__(self, arc):
        ScrollPanel.__init__(self, widget)
        self.setHeight(450)  # DON'T do this--it should be in a stylesheet
        self.setWidth(250)   # same advice, but set here to show the bars
        ### What is selected is highlighted, but you can't see it
        ### use a stylesheet and put within it:
        ### .gwt-TreeItem-selected {background-color: #5ff1f3;}
        ### in it to highlight the selection.
        tree = Tree()
        tree.addTreeListener(self)
        levels = [[] for i in range(MAXLEVELS)]
        ### For grins try levels = [[]]*MAXLEVELS 
        ### That will illustrate a limitation of the translator
        ### The pyhtonically equivalent lists built translate differently
        while arc:
            obj, level, = arc.pop(0)
            if level == 0:
                s1=self.createItem(obj.name, value=obj.name+" 0")
                s1.setState(True, fireEvents=False)
                levels[0].append(s1)
                continue
            if level>0 and level<MAXLEVELS+1:
                levels[level].append(levels[level-1][-1].addItem(
                       self.createItem(obj.name, value = 
                                       obj.name+loc(level,levels))))
                levels[level][-1].setState(False, fireEvents=False)
                continue
            else:
                raise BeyondDesignLimitError
        tree.addItem(s1)
        self.add(tree)


    def createItem(self, label, value=None):
        item = TreeItem(label)
        DOM.setStyleAttribute(item.getElement(), "cursor", "pointer")
        if value is not None:
            item.setUserObject(value)
        return item


    def onTreeItemSelected(self, item):
        value = item.getUserObject()
        specialvalue=value
        Window.alert("You clicked on " + str(value))


    def onTreeItemStateChanged(self, item):
        pass # We ignore this.


def traveler(node, index=0, blist=None):
    if blist is None:
        blist = []
    blist.append((node, index))
    if node.isLeaf():
        return
    index += 1
    for cnode in node.child:
        traveler(cnode, index=index, blist=blist)
    return blist

"""
    In a real app, traveler would be flattening an object tree and sending 
    NOTHING.  A routine would follow it to build two objects, 
              [{obj.name, level, str(uuid.uuid1())) for each traveler result], 
              sent via JSON and used as the (arc) parameter by TreeDemo
              {same str(uuid1()):object, ...} for the traveler objects (used 
              when an HTTPRequest() arrives...
              Together these objects and traveler will let you so stuff with 
              actual data trees as in an object database.
"""

def loc(level, levels):
    """ produces a string of the form 0.1.3.3 for the third sentence of the 
        third paragraph of the first subtopic of the root topic
    Params:
      level = an integer
      levels = a list of lists
    Returns:
      string
    Exceptions:
      None
    #### This is illustrative.  In a real app uuid.uuid1() would supply 
    #### values in the server side, and would be data key on what was selected
    #### when an action button was pressed.  
    """
    for i in range(level+1):
        if i == 0:
            result = ' 0'
            continue
        if i == level:
            result = '%s.%s' %(result, str(len(levels[i])+1))
            continue
        result = '%s.%s' %(result, str(len(levels[i])))
    return result

def main():
    root = RootPanel()
    """ The following test data is for demonstration purposes ..."""
    built222 = [TTS('performance_plot',[]), TTS('howler_algorithm', [])]
    built221 = [TTS('wf9',[])] 
    built123 = [TTS('sum_stats', [])]
    built122 = [TTS('sum_stats',[])]
    built121 = [TTS('scan_reads',[]),TTS('analyze_reads',[])]
    built023 = [TTS('wf7',[]), TTS('performance_plot',[])]
    built021 = [TTS('wf1',[])]
    built022 = [TTS('wf1',[]), TTS('scan_reads',[]), TTS('analyze reads',[])]
    built02 = [TTS('SNP calls',built021), 
               TTS('contrast_bwa/bismark', built022), 
               TTS('test_fidelity',built023)]
    built12 = [TTS('export',built121), TTS('fastq',built122), 
               TTS('bam', built123)]
    built22 = [TTS('Mosaik',built221), TTS('Measurements', built222)]
    blis1 = [TTS('Special_Alignment',built02), 
             TTS('Scan',built12),
             TTS('SureSelect', built22)]
    trroot = TTS(USER,blis1)
    """Traveler and its call should be in a method accessed via AJAX"""
    arc = traveler(trroot) # This should happen in a methodname.py
    t=TreeDemo(arc)
    root.add(t)
if __name__ == '__main__':
    main()
